@@ -4,7 +4,7 @@ use postgres_protocol::types;
 use self::bit_vec::BitVec;
 use std::error::Error;
 
-use types::{FromSql, ToSql, IsNull, Type, BIT, VARBIT};
+use types::{FromSql, IsNull, ToSql, Type, BIT, VARBIT};
 
 impl FromSql for BitVec {
     fn from_sql(_: &Type, raw: &[u8]) -> Result<BitVec, Box<Error + Sync + Send>> {
@@ -21,7 +21,7 @@ impl FromSql for BitVec {
 }
 
 impl ToSql for BitVec {
-    fn to_sql(&self, _: &Type, mut out: &mut Vec<u8>) -> Result<IsNull, Box<Error + Sync + Send>> {
+    fn to_sql(&self, _: &Type, out: &mut Vec<u8>) -> Result<IsNull, Box<Error + Sync + Send>> {
         types::varbit_to_sql(self.len(), self.to_bytes().into_iter(), out)?;
         Ok(IsNull::No)
     }
